@@ -4,8 +4,8 @@
 #include "options.h"
 
 #include <assert.h>
-#include <bits/stdint-uintn.h>
 #include <queue>
+#include <stdint.h>
 #include <string>
 
 class ResultReceiver {
